/*
  Copyright 2019 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.sample.common

import android.annotation.SuppressLint
import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.view.*
import androidx.appcompat.app.AppCompatActivity
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import androidx.viewpager2.widget.ViewPager2
import androidx.viewpager2.widget.ViewPager2.OnPageChangeCallback
import com.google.android.material.bottomsheet.BottomSheetDialogFragment
import com.google.android.material.tabs.TabLayout
import com.kakao.sdk.partner.model.KakaoPhase
import com.kakao.sdk.partner.sample.common.internal.PhaseAdapter
import com.kakao.sdk.partner.sample.common.internal.TabAdapter
import com.kakao.sdk.sample.common.internal.DebugFragment

class MainActivity : AppCompatActivity() {
    lateinit var tabAdapter: TabAdapter
    lateinit var viewPager: ViewPager2
    lateinit var bottomTab: TabLayout
    private val debugFragment = DebugFragment()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)

        viewPager = findViewById(R.id.viewPager)
        bottomTab = findViewById(R.id.bottomTab)

        tabAdapter = TabAdapter(this, this)

        viewPager.adapter = tabAdapter

        tabAdapter.fragments.map {
            bottomTab.newTab().setText(it.title).setContentDescription(it.title)
        }.forEach {
            bottomTab.addTab(it)
        }

        bottomTab.addOnTabSelectedListener(object : TabLayout.OnTabSelectedListener {
            override fun onTabReselected(tab: TabLayout.Tab?) {
            }

            override fun onTabUnselected(tab: TabLayout.Tab?) {
            }

            override fun onTabSelected(tab: TabLayout.Tab?) {
                viewPager.currentItem = tab!!.position
            }
        })

        viewPager.registerOnPageChangeCallback(object : OnPageChangeCallback() {
            override fun onPageSelected(position: Int) {
                bottomTab.getTabAt(position)?.select()
            }
        })
    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        if (debugFragment.isAdded) {
            menuInflater.inflate(R.menu.menu_debug_view, menu)
        } else {
            menuInflater.inflate(R.menu.menu_partner_main, menu)

            val preferences = getSharedPreferences("default", Context.MODE_PRIVATE)
            val currentPhase = KakaoPhase.fromName(
                preferences.getString("phase", KakaoPhase.PRODUCTION.phaseName) as String
            )
            val phaseButton = menu?.findItem(R.id.menu_phase)
            phaseButton?.title = currentPhase.phaseName
        }
        return true
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when (item.itemId) {
            R.id.menu_debug -> {
                if (debugFragment.isAdded) return true
                supportFragmentManager.beginTransaction().addToBackStack("main")
                    .add(R.id.container, debugFragment).commit()
                return true
            }

            R.id.menu_phase -> {
                showPhaseDialog()
            }

            R.id.menu_clear -> {
                debugFragment.clearLogs()
                return true
            }

            R.id.menu_close -> {
                if (!debugFragment.isAdded) return true
                supportFragmentManager.beginTransaction().remove(debugFragment).commit()
                return true
            }
        }
        return super.onOptionsItemSelected(item)
    }

    private fun showPhaseDialog() {
        val dialog = PhaseBottomSheet()
        dialog.show(supportFragmentManager, "phase")
    }
}

class PhaseBottomSheet : BottomSheetDialogFragment() {
    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?,
    ): View? = inflater.inflate(R.layout.bottom_sheet_phase, container, false)

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        val adapter = PhaseAdapter {
            changePhase(it)
        }
        view.findViewById<RecyclerView>(R.id.phaseRecyclerView).apply {
            this.layoutManager = LinearLayoutManager(context)
            this.adapter = adapter
        }
    }

    @SuppressLint("ApplySharedPref")
    private fun changePhase(phase: KakaoPhase) {
        val preferences = requireContext().getSharedPreferences("default", Context.MODE_PRIVATE)
        val currentPhase = preferences.getString("phase", KakaoPhase.PRODUCTION.phaseName)
        if (currentPhase == phase.phaseName) {
            dismiss()
            return
        }
        preferences.edit().putString("phase", phase.phaseName).commit()
        val intent = Intent(requireContext(), MainActivity::class.java)
            .addFlags(Intent.FLAG_ACTIVITY_NEW_TASK or Intent.FLAG_ACTIVITY_CLEAR_TASK)
        requireContext().startActivity(intent)
        Runtime.getRuntime().exit(0)
    }
}
